"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.unlistenQueueEvents = exports.listenQueueEvents = exports.connectToQueue = exports.setQueue = exports.getQueue = void 0;
const bullmq_1 = require("bullmq");
const utils_1 = require("./utils");
const ioredis_1 = __importDefault(require("ioredis"));
const fs_1 = __importDefault(require("fs"));
let queue;
let queueEvents;
async function getQueue() {
    if (!queue) {
        return utils_1.throwYellow("Need connect before");
    }
    await queue.waitUntilReady();
    return queue;
}
exports.getQueue = getQueue;
async function setQueue(name, prefix, options) {
    unlistenQueueEvents();
    queue && (await queue.close());
    const client = new ioredis_1.default(options);
    queue = new bullmq_1.Queue(name, { connection: client, ...{ prefix } });
    await queue.waitUntilReady();
    return queue;
}
exports.setQueue = setQueue;
async function connectToQueue({ queue: name, options }, vorpal) {
    var _a;
    const prefix = options.prefix || "bull";
    const host = options.host || "localhost";
    const port = options.port || 6379;
    const db = (_a = options.db) !== null && _a !== void 0 ? _a : 0;
    const password = options.password || void 0;
    const tls = options.cert
        ? {
            ca: fs_1.default.readFileSync(options.cert),
            rejectUnauthorized: false
        }
        : void 0;
    await setQueue(name, prefix, options.uri || {
        host,
        port,
        db,
        password,
        tls
    });
    vorpal.localStorage.setItem(utils_1.LAST_SAVED_CONNECTION_NAME, JSON.stringify({
        queue: name,
        options
    }));
    utils_1.logGreen(`Connected to ${host}:${port}, prefix: ${prefix}, queue: ${name}`);
    vorpal.delimiter(`BULL-REPL | ${prefix}.${name}> `).show();
}
exports.connectToQueue = connectToQueue;
function listenQueueEvents(queue) {
    if (queueEvents) {
        return;
    }
    queueEvents = new bullmq_1.QueueEvents(queue.name);
    queueEvents.on('completed', (job) => {
        utils_1.logBlue(`A job ${job.jobId} successfully completed with result: ${job.returnvalue}`);
    });
    queueEvents.on('delayed', (job) => {
        utils_1.logBlue(`A job ${job.jobId} delayed with delay ${job.delay}`);
    });
    queueEvents.on('drained', () => {
        utils_1.logBlue('Queue has processed all the waiting jobs');
    });
    queueEvents.on('failed', (job) => {
        utils_1.logYellow(`A job ${job.jobId} failed with error ${job.failedReason}`);
    });
    queueEvents.on('progress', (job, progress) => {
        utils_1.logBlue(`A job ${job.jobId} progress was updated to ${progress}`);
    });
    queueEvents.on('removed', (job) => {
        utils_1.logBlue(`A job ${job.jobId} was removed`);
    });
    queueEvents.on('stalled', (job) => {
        utils_1.logYellow(`A job ${job.jobId} has been marked as stalled`);
    });
    queueEvents.on('waiting', (job) => {
        utils_1.logBlue(`A job ${job.jobId} is waiting to be processed`);
    });
}
exports.listenQueueEvents = listenQueueEvents;
function unlistenQueueEvents() {
    if (!queueEvents) {
        return;
    }
    queueEvents.removeAllListeners();
    queueEvents = void 0;
}
exports.unlistenQueueEvents = unlistenQueueEvents;
