"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBootCommand = exports.wrapTryCatch = exports.splitJobsByFound = exports.throwYellow = exports.logBlue = exports.logYellow = exports.logGreen = exports.answer = exports.msLink = exports.jqLink = exports.logArray = exports.getTimeAgoFilter = exports.showJobs = exports.getJob = exports.LAST_SAVED_CONNECTION_NAME = void 0;
const node_jq_1 = require("node-jq");
const chalk_1 = __importDefault(require("chalk"));
const ms_1 = __importDefault(require("ms"));
const queue_1 = require("./queue");
exports.LAST_SAVED_CONNECTION_NAME = "_last-active";
exports.getJob = async (jobId) => {
    const queue = await queue_1.getQueue();
    const job = await queue.getJob(jobId);
    if (!job) {
        return exports.throwYellow(`Job "${jobId}" not found`);
    }
    return job;
};
exports.showJobs = async (arr, query) => {
    var _a, _b;
    const jobs = arr;
    const root = jobs
        .filter(j => j)
        .map(job => ({
        id: job.id,
        data: job.data,
        timestamp: Number.isNaN(job.timestamp)
            ? job.timestamp
            : new Date(job.timestamp),
        processedOn: job.processedOn && new Date(job.processedOn),
        finishedOn: job.finishedOn && new Date(job.finishedOn),
        name: job.name,
        failedReason: job.failedReason,
        stackTrace: job.stacktrace,
        returnValue: job.returnvalue,
        attemptsMade: job.attemptsMade,
        delay: job.delay,
        progress: job.progress
    }));
    const filteredData = query ? (await node_jq_1.run(query, { root }, { input: 'json', output: 'json' })) : { root };
    exports.logArray((_b = (_a = filteredData) === null || _a === void 0 ? void 0 : _a.root) !== null && _b !== void 0 ? _b : filteredData);
};
exports.getTimeAgoFilter = (timeAgo) => {
    return new Promise(resolve => {
        try {
            const msAgo = timeAgo && timeAgo.length ? ms_1.default(timeAgo) : void 0;
            const filter = msAgo ? `{root: [.root[] | select((.timestamp | strptime("%Y-%m-%dT%H:%M:%S.%3Z") | mktime | . * 1000) >= ${Date.now() - msAgo})]}` : '';
            resolve(filter);
        }
        catch (e) {
            exports.throwYellow(`Error: Argument to --timeAgo is invalid: ${e}`);
        }
    });
};
exports.logArray = (arr) => {
    console.dir(arr, {
        colors: true,
        depth: null,
        maxArrayLength: Infinity,
    });
    Array.isArray(arr) && console.log(`count: ${chalk_1.default.yellow(arr.length)}`);
};
exports.jqLink = "https://stedolan.github.io/jq/manual/#Basicfilters";
exports.msLink = "https://github.com/zeit/ms#examples";
exports.answer = async (vorpal, question) => {
    const answer = (await vorpal.activeCommand.prompt({
        name: "a",
        message: `${question}? (y/n): `
    }));
    if (answer.a !== "y") {
        exports.throwYellow("You cancel action");
    }
};
exports.logGreen = (msg) => {
    console.log(chalk_1.default.green(msg));
};
exports.logYellow = (msg) => {
    console.log(chalk_1.default.yellow(msg));
};
exports.logBlue = (msg) => {
    console.log(chalk_1.default.blueBright(msg));
};
exports.throwYellow = (msg) => {
    let err = new Error();
    err.yellow = true;
    err.stack = chalk_1.default.yellow(msg);
    throw err;
};
async function splitJobsByFound(jobIds) {
    const queue = await queue_1.getQueue();
    const jobs = await Promise.all(jobIds.map(id => queue.getJob(id)));
    let notFoundIds = [];
    let foundJobs = [];
    let i = 0;
    for (const jobId of jobIds) {
        const job = jobs[i];
        if (job) {
            foundJobs.push(job);
        }
        else {
            notFoundIds.push(jobId);
        }
        i++;
    }
    return { notFoundIds, foundJobs };
}
exports.splitJobsByFound = splitJobsByFound;
function wrapTryCatch(fn) {
    return async function (args) {
        try {
            return await fn.call(this, args);
        }
        catch (e) {
            if (e.yellow) {
                throw e;
            }
            return exports.throwYellow(e.message);
        }
    };
}
exports.wrapTryCatch = wrapTryCatch;
function getBootCommand(vorpal) {
    const parsed = vorpal.parse(process.argv, {
        use: "minimist"
    });
    if (Array.isArray(parsed._)) {
        return parsed._.join(" ");
    }
    return "";
}
exports.getBootCommand = getBootCommand;
